function strim (s)
	return (string.gsub(s, "^%s*(.-)%s*$", "%1"))
end

function GenerateUniqueContainerName(containers, startWith)
	local exists = true
	local count = 1
	local name

	while (exists == true) do
		name = startWith..string.format("%.0f", count)
		local match = false
		for _, container in ipairs(containers) do
			if(container.options.name == name) then
				match = true
			end
		end
		
		if(match == false) then
			exists = false
		end
		count = count + 1
	end
	return name
end

function deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

do
	local flag = {}
	
	function CloneTable(tbl, cloneV, cloneK)
		assert(not flag[tbl], "CloneTable(): cannot deep-clone a table that contains a reference to itself")
		flag[tbl] = 1
		local newtbl = {}
		for k,v in pairs(tbl) do
			if (cloneK and type(k)=="table") then k = CloneTable(k, cloneV, cloneK) end
			if (cloneV and type(v)=="table") then v = CloneTable(v, cloneV, cloneK) end
			newtbl[k] = v
		end
		flag[tbl] = nil
		return newtbl
	end -- __cloneTable()
	
end

function RotateTexture(self, angle)
	local function GetCorner(angle)
		local Root2 = 2 ^ 0.5;
		return 0.5 + cos( angle ) / Root2, 0.5 + sin( angle ) / Root2;
	end

	local LRx, LRy = GetCorner( angle + 45 );
	local LLx, LLy = GetCorner( angle + 135 );
	local ULx, ULy = GetCorner( angle - 135 );
	local URx, URy = GetCorner( angle - 45 );
	self:SetTexCoord( ULx, ULy, LLx, LLy, URx, URy, LRx, LRy );
end

function Round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

function SecondsToBlizz(sSeconds)
	local nSeconds = tonumber(sSeconds)
	if nSeconds == 0 then
		return "0s";
	else
		hours = math.ceil(nSeconds/3600)
		mins = math.ceil(nSeconds/60)
		secs = math.ceil(nSeconds);
		
		if(nSeconds > 3600) then
			return tostring(hours).."h"
		end
		
		if(nSeconds > 60) then
			return tostring(mins).."m"
		end
		
		return tostring(math.floor(nSeconds)).."s"
	end
end

function SecondsToClock(sSeconds)
	local nSeconds = tonumber(sSeconds)
	if nSeconds == 0 then
		return "0";
	else
		nHours = string.format("%02.f", math.floor(nSeconds/3600));
		nMins = string.format("%02.f", math.floor(nSeconds/60 - (nHours*60)));
		nSecs = string.format("%02.f", math.floor(nSeconds - nHours*3600 - nMins *60));
		
		if(nSeconds > 3600) then
			return nHours..":"..nMins..":"..nSecs
		end
		
		if(nSeconds > 60) then
			return nMins..":"..nSecs
		end
		
		return Round(nSeconds)
	end
end

function GetTableCount(tbl)
	local cnt = 0
	for k, v in pairs(tbl) do
		cnt = cnt + 1
	end
	return cnt
end